@@ -19,23 +19,26 @@
 
 #include <sys/types.h>
 #include <dirent.h>
+#include <stdio.h>
 
 #include "DB/ImageDB.h"
 #include "DB/ResultId.h"
+#include "DB/Result.h"
 
 #include <qfileinfo.h>
+#include <QStringList>
 #include <QProgressDialog>
 #include <klocale.h>
 #include <qapplication.h>
 #include <qeventloop.h>
 #include <kmessagebox.h>
+#include "DB/MD5Map.h"
 
 #include "config-kpa-exiv2.h"
 #ifdef HAVE_EXIV2
 #  include "Exif/Database.h"
 #endif
 
-#include "Browser/BrowserWidget.h"
 #include "ImageManager/Manager.h"
 #include "ImageManager/RawImageDecoder.h"
 #include "Settings/SettingsData.h"
@@ -64,7 +67,6 @@ bool NewImageFinder::findImages()
 
     // To avoid deciding if the new images are shown in a given thumbnail view or in a given search
     // we rather just go to home.
-    Browser::BrowserWidget::instance()->home();  // TODO: DEPENDENCY: DB:: should not depend on other directories
     return (!_pendingLoad.isEmpty()); // returns if new images was found.
 }
 
@@ -134,14 +136,16 @@ void NewImageFinder::searchForNewFiles( const QSet<QString>& loadedFiles, QStrin
     FastDir dir( directory );
     QStringList dirList = dir.entryList( );
     ImageManager::RAWImageDecoder dec;   // TODO: DEPENDENCY: DB:: should not reference other directories
+    QStringList excluded;
+    excluded << Settings::SettingsData::instance()->excludeDirectories();
+    excluded = excluded.at(0).split(QString::fromLatin1(","));
     for( QStringList::const_iterator it = dirList.constBegin(); it != dirList.constEnd(); ++it ) {
         QString file = directory + QString::fromLatin1("/") + *it;
-        if ( (*it) == QString::fromLatin1(".") || (*it) == QString::fromLatin1("..") ||
-             (*it) == QString::fromLatin1("ThumbNails") ||
-             (*it) == QString::fromLatin1("CategoryImages") ||
-	         loadedFiles.contains( file ) ||
-	         dec._skipThisFile(loadedFiles, file) )
-            continue;
+	if ( (*it) == QString::fromLatin1(".") || (*it) == QString::fromLatin1("..") ||
+                excluded.contains( (*it) ) || loadedFiles.contains( file ) ||
+                dec._skipThisFile(loadedFiles, file) ||
+                (*it) == QString::fromLatin1("CategoryImages") )
+	    continue;
 
         QFileInfo fi( file );
 
@@ -181,8 +185,10 @@ void NewImageFinder::loadExtraFiles()
         if ( dialog.wasCanceled() )
             return;
         ImageInfoPtr info = loadExtraFile( (*it).first, (*it).second );
-        if ( info )
+        if ( info ) {
+            markUnTagged(info);
             newImages.append(info);
+        }
     }
     DB::ImageDB::instance()->addImages( newImages );
 }
@@ -225,11 +231,77 @@ ImageInfoPtr NewImageFinder::loadExtraFile( const QString& relativeNewFileName,
         }
     }
 
+    QString err = Settings::SettingsData::instance()->modifiedFileComponent();
+    QRegExp modifiedFileComponent =
+        QRegExp(Settings::SettingsData::instance()->modifiedFileComponent());
+    
+    // check to see if this is a new version of a previous image
+    ImageInfoPtr info = ImageInfoPtr(new ImageInfo( relativeNewFileName, type ));
+    ImageInfoPtr originalInfo;
+    QString originalFileName;
+
+    if (Settings::SettingsData::instance()->detectModifiedFiles()) {
+        // should be cached because loading once per image is expensive
+        QString err = Settings::SettingsData::instance()->modifiedFileComponent();
+        QRegExp modifiedFileComponent =
+            QRegExp(Settings::SettingsData::instance()->modifiedFileComponent());
+        // requires at least *something* in the modifiedFileComponent
+        if (err.length() >= 0 &&
+            relativeNewFileName.contains(modifiedFileComponent)) {
+
+            originalFileName = relativeNewFileName;
+            QString originalFileComponent =
+                Settings::SettingsData::instance()->originalFileComponent();
+            originalFileName.replace(modifiedFileComponent, originalFileComponent);
+
+            MD5 originalSum = Utilities::MD5Sum( Utilities::absoluteImageFileName( originalFileName ) );
+            if ( DB::ImageDB::instance()->md5Map()->contains( originalSum ) ) {
+                // we have a previous copy of this file; copy it's data
+                // from the original.
+                originalInfo = DB::ImageDB::instance()->info( originalFileName, DB::RelativeToImageRoot );
+                if ( !originalInfo ) {
+                    qWarning("How did that happen? We couldn't find info for the original image %s; can't copy the original data to %s", qPrintable(originalFileName), qPrintable(relativeNewFileName));
+                } else {
+                    info->copyExtraData(*originalInfo);
+                }
+
+                /* if requested to move, then delete old data from original */
+                if (Settings::SettingsData::instance()->moveOriginalContents() ) {
+                    originalInfo->removeExtraData();
+                }
+            }
+        }
+    }
+
     // also inserts image into exif db if present:
-    ImageInfoPtr info = ImageInfoPtr( new ImageInfo( relativeNewFileName, type ) );
     info->setMD5Sum(sum);
     DB::ImageDB::instance()->md5Map()->insert( sum, info->fileName(DB::RelativeToImageRoot) );
 
+    if (originalInfo &&
+        Settings::SettingsData::instance()->autoStackNewFiles() ) {
+        // we have to do this immediately to get the ids
+        ImageInfoList newImages;
+        markUnTagged(info);
+        newImages.append(info);
+        DB::ImageDB::instance()->addImages( newImages );
+
+        // stack the files together
+        DB::ResultId olderfile = DB::ImageDB::instance()->ID_FOR_FILE(originalFileName);
+        DB::ResultId newerfile = DB::ImageDB::instance()->ID_FOR_FILE(info->fileName(DB::AbsolutePath));
+        DB::Result tostack = DB::Result();
+
+        tostack.append(newerfile);
+        tostack.append(olderfile);
+        DB::ImageDB::instance()->stack(tostack);
+
+        // ordering: XXX we ideally want to place the new image right
+        // after the older one in the list.
+
+        // XXX: deal with already-stacked items; currently a silent fail
+
+        info = NULL;  // we already added it, so don't process again
+    }
+    
     return info;
 }
 
@@ -290,3 +362,11 @@ bool  NewImageFinder::calculateMD5sums(
 
     return dirty;
 }
+
+void DB::NewImageFinder::markUnTagged( ImageInfoPtr info )
+{
+    if ( Settings::SettingsData::instance()->hasUntaggedCategoryFeatureConfigured() ) {
+        info->addCategoryInfo( Settings::SettingsData::instance()->untaggedCategory(),
+                           Settings::SettingsData::instance()->untaggedTag() );
+    }
+}
