@@ -17,38 +17,25 @@
 */
 
 #include "DateBarWidget.h"
+#include <DB/ImageDateCollection.h>
+#include <QMenu>
 #include <qdatetime.h>
 #include <qpainter.h>
 #include <qfontmetrics.h>
-//Added by qt3to4:
 #include <QContextMenuEvent>
-#include <QKeyEvent>
-#include <QResizeEvent>
-#include <QFocusEvent>
-#include <QMouseEvent>
-#include <QPaintEvent>
-#include <QWheelEvent>
-#include "ViewHandler.h"
 #include <qtoolbutton.h>
 #include <qaction.h>
-#include <kglobal.h>
-#include <kiconloader.h>
 #include <math.h>
 #include <klocale.h>
 #include "Settings/SettingsData.h"
-#include <qtooltip.h>
 #include <KIcon>
+#include "MouseHandler.h"
 
 const int borderAboveHistogram = 4;
 const int borderArroundWidget = 0;
 const int buttonWidth = 22;
 const int arrowLength = 20;
 
-/**
- * \namespace DateBar
- * \brief The date bar at the bottom of the main window
- */
-
 /**
  * \class DateBar::DateBarWidget
  * \brief This class represents the date bar at the bottom of the main window.
@@ -126,9 +113,6 @@ void DateBar::DateBarWidget::redraw()
     if ( _buffer.isNull() )
         return;
 
-    if (_dates.isNull() )
-        return;
-
     QPainter p( &_buffer );
     p.setRenderHint( QPainter::Antialiasing );
     p.setFont( font() );
@@ -139,6 +123,9 @@ void DateBar::DateBarWidget::redraw()
     p.setBrush( palette().brush( QPalette::Background ) );
     p.drawRect( rect() );
 
+    if (_dates.isNull() )
+        return;
+
     // Draw the area with histograms
     QRect barArea = barAreaGeometry();
 
