@@ -17,6 +17,7 @@
 */
 
 #include "HTMLDialog.h"
+#include <QComboBox>
 
 #include <klocale.h>
 #include <qlayout.h>
@@ -32,16 +33,13 @@
 #include <kmessagebox.h>
 #include <kfileitem.h>
 #include <kio/netaccess.h>
-#include <q3textedit.h>
-#include <kdebug.h>
-#include <kglobal.h>
-#include <kiconloader.h>
 #include "MainWindow/Window.h"
 #include "DB/CategoryCollection.h"
 #include "DB/ImageDB.h"
 #include "Generator.h"
 #include "ImageSizeCheckBox.h"
 #include <QTextEdit>
+#include <QStringMatcher>
 using namespace HTMLGenerator;
 
 
@@ -77,20 +75,29 @@ void HTMLDialog::createContentPage()
     label->setBuddy( _title );
     lay2->addWidget( _title, 0, 1 );
 
+    // Copyright
+    label = new QLabel( i18n("Copyright:"), contentPage );
+    label->setAlignment( Qt::AlignTop );
+    lay2->addWidget( label, 1, 0 );
+    _copyright = new KLineEdit( contentPage );
+    _copyright->setText( Settings::SettingsData::instance()->HTMLCopyright() );
+    label->setBuddy( _copyright );
+    lay2->addWidget( _copyright, 1, 1 );
+
     // Description
     label = new QLabel( i18n("Description:"), contentPage );
     label->setAlignment( Qt::AlignTop );
-    lay2->addWidget( label, 1, 0 );
+    lay2->addWidget( label, 2, 0 );
     _description = new QTextEdit( contentPage );
     label->setBuddy( _description );
-    lay2->addWidget( _description, 1, 1 );
+    lay2->addWidget( _description, 2, 1 );
 
     _generateKimFile = new QCheckBox( i18n("Create .kim export file"), contentPage );
-    _generateKimFile->setChecked( true );
+    _generateKimFile->setChecked( Settings::SettingsData::instance()->HTMLKimFile() );
     lay1->addWidget( _generateKimFile );
 
     _inlineMovies = new QCheckBox( i18n( "Inline Movies in pages" ), contentPage );
-    _inlineMovies->setChecked( true );
+    _inlineMovies->setChecked( Settings::SettingsData::instance()->HTMLInlineMovies() );
     lay1->addWidget( _inlineMovies );
 
     // What to include
@@ -102,17 +109,28 @@ void HTMLDialog::createContentPage()
     _whatToIncludeMap.insert( QString::fromLatin1("**DESCRIPTION**"), cb );
     lay3->addWidget( cb, 0, 0 );
 
-    int row=0;
-    int col=0;
+    _date = new QCheckBox( i18n("Date"), whatToInclude );
+    _date->setChecked( Settings::SettingsData::instance()->HTMLDate() );
+    _whatToIncludeMap.insert( QString::fromLatin1("**DATE**"), _date );
+    lay3->addWidget( _date, 0, 1 );
 
-    Q3ValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
-    for( Q3ValueList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it ) {
+    int row=1;
+    int col=0;
+    QString selectionsTmp = Settings::SettingsData::instance()->HTMLIncludeSelections();
+    QStringMatcher* pattern = new QStringMatcher();
+    pattern->setPattern(QString::fromLatin1("**DESCRIPTION**"));
+    cb->setChecked( pattern->indexIn (selectionsTmp)  >= 0 ? 1 : 0 );
+
+     QList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
+     for( QList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it ) {
         if ( ! (*it)->isSpecialCategory() ) {
-            if ( ++col % 2 == 0 )
-                ++row;
             QCheckBox* cb = new QCheckBox( (*it)->text(), whatToInclude );
             lay3->addWidget( cb, row, col%2 );
             _whatToIncludeMap.insert( (*it)->name(), cb );
+	    pattern->setPattern((*it)->name());
+	    cb->setChecked( pattern->indexIn (selectionsTmp)  >= 0 ? 1 : 0 );
+            if ( ++col % 2 == 0 )
+                ++row;
         }
     }
 }
@@ -139,7 +157,7 @@ void HTMLDialog::createLayoutPage()
     _thumbSize = new QSpinBox;
     _thumbSize->setRange( 16, 256 );
 
-    _thumbSize->setValue( 128 );
+    _thumbSize->setValue( Settings::SettingsData::instance()->HTMLThumbSize() );
     lay3->addWidget( _thumbSize );
     lay3->addStretch(1);
     label->setBuddy( _thumbSize );
@@ -155,7 +173,7 @@ void HTMLDialog::createLayoutPage()
 
     label->setBuddy( _numOfCols);
 
-    _numOfCols->setValue( 5 );
+    _numOfCols->setValue( Settings::SettingsData::instance()->HTMLNumOfCols() );
     lay4->addWidget( _numOfCols );
     lay4->addStretch( 1 );
 
@@ -196,7 +214,24 @@ void HTMLDialog::createLayoutPage()
         lay5->addWidget( sizeOrig, row, ++col );
     }
 
-    size800->setChecked( 1 );
+    QString tmp;
+    if ((tmp = Settings::SettingsData::instance()->HTMLSizes()) != QString::fromLatin1("")) {
+	QStringMatcher* pattern = new QStringMatcher(QString::fromLatin1("320"));
+	size320->setChecked( pattern->indexIn (tmp) >= 0 ? 1 : 0);
+	pattern->setPattern(QString::fromLatin1("640"));
+	size640->setChecked( pattern->indexIn (tmp) >= 0 ? 1 : 0);
+	pattern->setPattern(QString::fromLatin1("800"));
+	size800->setChecked( pattern->indexIn (tmp)  >= 0 ? 1 : 0 );
+	pattern->setPattern(QString::fromLatin1("1024"));
+	size1024->setChecked( pattern->indexIn (tmp)  >= 0 ? 1 : 0);
+	pattern->setPattern(QString::fromLatin1("1280"));
+	size1280->setChecked( pattern->indexIn (tmp)  >= 0 ? 1 : 0);
+	pattern->setPattern(QString::fromLatin1("1600"));
+	size1600->setChecked( pattern->indexIn (tmp)  >= 0 ? 1 : 0);
+	pattern->setPattern(QString::fromLatin1("-1"));
+	sizeOrig->setChecked( pattern->indexIn (tmp)  >= 0 ? 1 : 0);
+    } else
+	size800->setChecked( 1 );
 
     _cbs << size800 << size1024 << size1280 << size640 << size1600 << size320 << sizeOrig;
 
@@ -278,6 +313,15 @@ void HTMLDialog::slotOk()
     Settings::SettingsData::instance()->setHTMLBaseDir( _baseDir->text() );
     Settings::SettingsData::instance()->setHTMLBaseURL( _baseURL->text() );
     Settings::SettingsData::instance()->setHTMLDestURL( _destURL->text() );
+    Settings::SettingsData::instance()->setHTMLCopyright( _copyright->text() );
+    Settings::SettingsData::instance()->setHTMLDate( _date->isChecked() );
+    Settings::SettingsData::instance()->setHTMLTheme( _themeBox->currentIndex() );
+    Settings::SettingsData::instance()->setHTMLKimFile( _generateKimFile->isChecked() );
+    Settings::SettingsData::instance()->setHTMLInlineMovies( _inlineMovies->isChecked() );
+    Settings::SettingsData::instance()->setHTMLThumbSize( _thumbSize->value() );
+    Settings::SettingsData::instance()->setHTMLNumOfCols( _numOfCols->value() );
+    Settings::SettingsData::instance()->setHTMLSizes( activeSizes() );
+    Settings::SettingsData::instance()->setHTMLIncludeSelections( includeSelections() );
 
     Generator generator( setup(), this );
     generator.generate();
@@ -338,8 +382,8 @@ bool HTMLDialog::checkVars()
     if ( exists ) {
         int answer = KMessageBox::warningYesNo( this,
                                                 i18n("<p>Output directory %1 already exists. "
-                                                     "Usually you should specify a new directory.</p>"
-                                                     "<p>Should I delete %2 first?</p>").arg( outputDir ).arg( outputDir ),
+                                                     "Usually, this means you should specify a new directory.</p>"
+                                                     "<p>Should %2 be deleted first?</p>").arg( outputDir ).arg( outputDir ),
                                                 i18n("Directory Exists"), KStandardGuiItem::yes(), KStandardGuiItem::no(),
                                                 QString::fromLatin1("html_export_delete_original_directory") );
         if ( answer == KMessageBox::Yes ) {
@@ -361,10 +405,42 @@ QList<ImageSizeCheckBox*> HTMLDialog::activeResolutions() const
     return res;
 }
 
+QString HTMLDialog::activeSizes() const
+{
+    QString res;
+    for( QList<ImageSizeCheckBox*>::ConstIterator sizeIt = _cbs.begin(); sizeIt != _cbs.end(); ++sizeIt ) {
+        if ( (*sizeIt)->isChecked() ) {
+	    if (res.length() > 0)
+		res.append(QString::fromLatin1(","));
+            res.append(QString::number((*sizeIt)->width()));
+	}
+    }
+    return res;
+}
+
+QString HTMLDialog::includeSelections() const
+{
+    QString sel;
+    Setup _setup = setup();
+
+    for( QMap<QString,QCheckBox*>::ConstIterator it = _whatToIncludeMap.begin();
+         it != _whatToIncludeMap.end(); ++it ) {
+	QString name = it.key();
+        if ( _setup.includeCategory(name) ) {
+	    if (sel.length() > 0)
+		sel.append(QString::fromLatin1(","));
+            sel.append(name);
+	}
+    }
+    return sel;
+}
+
 void HTMLDialog::populateThemesCombo()
 {
     QStringList dirs = KGlobal::dirs()->findDirs( "data", QString::fromLocal8Bit("kphotoalbum/themes/") );
     int i = 0;
+    int theme = 0;
+    int defaultthemes = 0;
     for(QStringList::Iterator it = dirs.begin(); it != dirs.end(); ++it) {
         QDir dir(*it);
         QStringList themes = dir.entryList( QDir::Dirs | QDir::Readable );
@@ -376,16 +452,29 @@ void HTMLDialog::populateThemesCombo()
             KConfigGroup config = themeconfig.group("theme");
             QString themeName = config.readEntry( "Name" );
             QString themeAuthor = config.readEntry( "Author" );
+            QString themeDefault = config.readEntry( "Default" );
 
             enableButtonOk( true );
             _themeBox->insertItem( i, i18n( "%1 (by %2)",themeName, themeAuthor ) );
             _themes.insert( i, themePath );
+
+            if (themeDefault == QString::fromLatin1("true")) {
+		theme = i;
+		defaultthemes++;
+            }
             i++;
         }
     }
     if(_themeBox->count() < 1) {
         KMessageBox::error( this, i18n("Could not find any themes - this is very likely an installation error" ) );
     }
+    if (Settings::SettingsData::instance()->HTMLTheme() >= 0)
+	_themeBox->setCurrentIndex( Settings::SettingsData::instance()->HTMLTheme() );
+    else {
+	_themeBox->setCurrentIndex( theme );
+	if (defaultthemes > 1)
+	    KMessageBox::information( this, i18n("More than one theme is set as default, using theme %1", _themeBox->currentText()) );
+    }
 }
 
 int HTMLDialog::exec(const DB::Result& list)
@@ -405,6 +494,8 @@ Setup HTMLGenerator::HTMLDialog::setup() const
     setup.setDestURL( _destURL->text() );
     setup.setOutputDir( _outputDir->text() );
     setup.setThumbSize( _thumbSize->value() );
+    setup.setCopyright( _copyright->text() );
+    setup.setDate( _date->isChecked() );
     setup.setDescription( _description->toPlainText() );
     setup.setNumOfCols( _numOfCols->value() );
     setup.setGenerateKimFile( _generateKimFile->isChecked() );
