@@ -21,10 +21,8 @@
 
 #include <qdialog.h>
 #include <qimage.h>
-//Added by qt3to4:
-#include <QCloseEvent>
-#include <QLabel>
 #include "DB/ImageInfoPtr.h"
+
 class QCloseEvent;
 class QLabel;
 
