@@ -19,14 +19,15 @@
 #ifndef DELETEDIALOG_H
 #define DELETEDIALOG_H
 #include <KDialog>
-//Added by qt3to4:
 #include <QLabel>
-
+#include <kjob.h>
+#include <qradiobutton.h>
 #include "DB/Result.h"
 
 class QLabel;
 class QCheckBox;
-
+class KJob;
+namespace KIO {class Job; }
 namespace MainWindow
 {
 
@@ -39,11 +40,14 @@ public:
 
 protected slots:
     void deleteImages();
+    void slotKIOJobCompleted( KJob* );
 
 private:
     DB::Result _list;
     QLabel* _label;
-    QCheckBox* _delete_file;
+    QRadioButton* _deleteFile;
+    QRadioButton* _useTrash;
+    QRadioButton* _deleteFromDb;
 
 };
 
