@@ -18,7 +18,6 @@
 #ifndef SEARCHBAR_H
 #define SEARCHBAR_H
 #include <ktoolbar.h>
-//Added by qt3to4:
 #include <QEvent>
 class KLineEdit;
 class KMainWindow;
@@ -42,8 +41,7 @@ public slots:
 signals:
     void textChanged( const QString& );
     void returnPressed();
-    void scrollLine( int direction );
-    void scrollPage( int direction );
+    void keyPressed( QKeyEvent* );
 
 private:
     KLineEdit* _edit;
