@@ -476,7 +476,7 @@ void MainWindow::Window::slotDeleteSelected()
 void MainWindow::Window::slotCopySelectedURLs()
 {
     KUrl::List urls;
-    Q_FOREACH(const DB::ImageInfoPtr info, selectedOnDisk().fetchInfos()) {
+    Q_FOREACH(const DB::ImageInfoPtr info, selected().fetchInfos()) {
         const QString fileName = info->fileName(DB::AbsolutePath);
         urls.append( fileName );
     }
@@ -574,7 +574,8 @@ void MainWindow::Window::launchViewer(const DB::Result& inputMediaList, bool reu
         viewer->activateWindow();
     }
     else
-        viewer = new Viewer::ViewerWidget;
+        viewer = new Viewer::ViewerWidget(Viewer::ViewerWidget::ViewerWindow,
+                                          &_viewerInputMacros);
     connect( viewer, SIGNAL( rotated() ), this, SLOT( reloadThumbnailsAndFlushCache() ) );
     connect( viewer, SIGNAL( soughtTo(const DB::ResultId&) ), _thumbnailView, SLOT( changeSingleSelection(const DB::ResultId&) ) );
 
@@ -1482,6 +1483,8 @@ void MainWindow::Window::slotSelectionChanged( int count )
 {
 #ifdef HASKIPI
     _pluginInterface->slotSelectionChanged( count != 0 );
+#else
+    Q_UNUSED( count );
 #endif
 }
 
