@@ -24,7 +24,6 @@
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QDir>
-#include <QPixmap> //Added by qt3to4
 #include <QPixmapCache>
 #include <QStringList>
 
@@ -156,12 +155,15 @@ property_copy( useEXIFRotate         , setUseEXIFRotate         , bool
 property_copy( useEXIFComments       , setUseEXIFComments       , bool          , General, true                       )
 property_copy( searchForImagesOnStart, setSearchForImagesOnStart, bool          , General, true                       )
 property_copy( skipRawIfOtherMatches , setSkipRawIfOtherMatches , bool          , General, false                      )
+property_copy( useRawThumbnail       , setUseRawThumbnail       , bool          , General, false                      )
+property_copy( useRawThumbnailSize   , setUseRawThumbnailSize   , QSize         , General, QSize(1024,768)            )
 property_copy( useCompressedIndexXML , setUseCompressedIndexXML , bool          , General, false                      )
 property_copy( compressBackup        , setCompressBackup        , bool          , General, true                       )
 property_copy( showSplashScreen      , setShowSplashScreen      , bool          , General, true                       )
 property_copy( autoSave              , setAutoSave              , int           , General, 5                          )
 property_copy( backupCount           , setBackupCount           , int           , General, 5                          )
 property_enum( tTimeStamps           , setTTimeStamps           , TimeStampTrust, General, Always                     )
+property_copy( excludeDirectories    , setExcludeDirectories    , QString       , General, QString::fromLatin1("xml,ThumbNails,.thumbs") )
 
 getValueFunc( QSize,histogramSize,  General,QSize(15,30) )
 getValueFunc( ViewSortType,viewSortType,  General,(int)SortLastUse )
@@ -211,6 +213,18 @@ bool SettingsData::trustTimeStamps()
     }
 }
 
+////////////////////////////////
+//// File Version Detection ////
+////////////////////////////////
+
+property_copy( detectModifiedFiles   , setDetectModifiedFiles   , bool          , FileVersionDetection, false               )
+property_copy( modifiedFileComponent , setModifiedFileComponent , QString       , FileVersionDetection, QString()           )
+property_copy( originalFileComponent , setOriginalFileComponent , QString       , FileVersionDetection, QString()           )
+property_copy( moveOriginalContents  , setMoveOriginalContents  , bool          , FileVersionDetection, false               )
+property_copy( autoStackNewFiles     , setAutoStackNewFiles     , bool          , FileVersionDetection, true                )
+property_copy( copyFileComponent     , setCopyFileComponent     , QString       , FileVersionDetection, QString()           )
+property_copy( copyFileReplacementComponent , setCopyFileReplacementComponent , QString  , FileVersionDetection, QString()  )
+
 ////////////////////
 //// Thumbnails ////
 ////////////////////
@@ -219,13 +233,14 @@ property_copy( displayLabels           , setDisplayLabels          , bool
 property_copy( displayCategories       , setDisplayCategories      , bool                , Thumbnails, false      )
 property_copy( autoShowThumbnailView   , setAutoShowThumbnailView  , bool                , Thumbnails, 0          )
 property_copy( showNewestThumbnailFirst, setShowNewestFirst        , bool                , Thumbnails, false      )
-property_copy( thumbnailDarkBackground , setThumbnailDarkBackground, bool                , Thumbnails, true       )
 property_copy( thumbnailDisplayGrid    , setThumbnailDisplayGrid   , bool                , Thumbnails, false      )
 property_copy( previewSize             , setPreviewSize            , int                 , Thumbnails, 256        )
-property_copy( thumbnailSpace          , setThumbnailSpace         , int                 , Thumbnails, 1          )
+property_copy( thumbnailSpace          , setThumbnailSpace         , int                 , Thumbnails, 4          )
 property_enum( thumbnailAspectRatio    , setThumbnailAspectRatio   , ThumbnailAspectRatio, Thumbnails, Aspect_4_3 )
+property_ref ( thumbnailFormat         , setThumbnailFormat        , QString             , Thumbnails, QString::fromLatin1("ppm") )
+property_ref(  backgroundColor         , setBackgroundColor        , QString             , Thumbnails, QColor(Qt::darkGray).name() )
 
-getValueFunc( int,thumbSize,  Thumbnails,128)
+getValueFunc( int,thumbSize,  Thumbnails, 150)
 
 getValueFunc( int,thumbnailCacheScreens,  Thumbnails,3) // Three pages sounds good; one before, one after the current screen
 
@@ -306,66 +321,8 @@ QString SettingsData::albumCategory() const
     return category;
 }
 
-// PENDING(blackie) move this function to Category
-QString SettingsData::fileForCategoryImage( const QString& category, QString member ) const
-{
-    QString dir = imageDirectory() + STR("CategoryImages" );
-    member.replace( QChar::fromLatin1(' '), QChar::fromLatin1('_') );
-    member.replace( QChar::fromLatin1('/'), QChar::fromLatin1('_') );
-    QString fileName = dir + STR("/%1-%2.jpg").arg( category ).arg( member );
-    return fileName;
-}
-
-// PENDING(blackie) moved this function to Category
-QPixmap SettingsData::categoryImage( const QString& category, QString member, int size ) const
-{
-    QString fileName = fileForCategoryImage( category, member );
-    QString key = STR( "%1-%2" ).arg(size).arg(fileName);
-    QPixmap res;
-    if ( QPixmapCache::find( key, res ) )
-        return res;
-
-    QImage img;
-    bool ok = img.load( fileName, "JPEG" );
-    if ( ! ok ) {
-        if ( DB::ImageDB::instance()->memberMap().isGroup( category, member ) )
-            img = KIconLoader::global()->loadIcon( STR( "kuser" ), KIconLoader::Desktop, size ).toImage();
-        else
-            img = DB::ImageDB::instance()->categoryCollection()->categoryForName( category )->icon().toImage();
-    }
-    res = QPixmap::fromImage( Utilities::scaleImage(img, size, size, Qt::KeepAspectRatio) );
-
-    QPixmapCache::insert( key, res );
-    return res;
-}
-
-// PENDING(blackie) move this function to Category
-void SettingsData::setCategoryImage( const QString& category, QString member, const QImage& image )
-{
-    QString dir = imageDirectory() + STR("CategoryImages" );
-    QFileInfo fi( dir );
-    bool ok;
-    if ( !fi.exists() ) {
-        bool ok = QDir().mkdir( dir );
-        if ( !ok ) {
-            KMessageBox::error( 0, i18n("Unable to create directory '%1'.", dir ), i18n("Unable to Create Directory") );
-            return;
-        }
-    }
-    QString fileName = fileForCategoryImage( category, member );
-    ok = image.save( fileName, "JPEG" );
-    if ( !ok ) {
-        KMessageBox::error( 0, i18n("Error when saving image '%1'.",fileName), i18n("Error Saving Image") );
-        return;
-    }
-
-    // PENDING(blackie) HACK ALERT: Remove all images rather than just these resolutions.
-    QString key = STR( "64-%2" ).arg(fileName);
-    QPixmapCache::remove( key );
-
-    key = STR( "128-%2" ).arg(fileName);
-    QPixmapCache::remove( key );
-}
+property_ref( untaggedCategory, setUntaggedCategory, QString, General, "Events")
+property_ref( untaggedTag,      setUntaggedTag,      QString, General, "untagged")
 
 //////////////
 //// Exif ////
@@ -418,6 +375,42 @@ property_ref_(
         HTMLDestURL, setHTMLDestURL, QString,
         groupForDatabase( "HTML Settings" ),
         STR( "file://" ) + HTMLBaseDir() )
+property_ref_(
+        HTMLCopyright, setHTMLCopyright, QString,
+        groupForDatabase( "HTML Settings" ),
+        STR( "" ) )
+property_ref_(
+        HTMLDate, setHTMLDate, int,
+        groupForDatabase( "HTML Settings" ),
+        true )
+property_ref_(
+        HTMLTheme, setHTMLTheme, int,
+        groupForDatabase( "HTML Settings" ),
+        -1 )
+property_ref_(
+        HTMLKimFile, setHTMLKimFile, int,
+        groupForDatabase( "HTML Settings" ),
+        true )
+property_ref_(
+        HTMLInlineMovies, setHTMLInlineMovies, int,
+        groupForDatabase( "HTML Settings" ),
+        true )
+property_ref_(
+        HTMLThumbSize, setHTMLThumbSize, int,
+        groupForDatabase( "HTML Settings" ),
+        128 )
+property_ref_(
+        HTMLNumOfCols, setHTMLNumOfCols, int,
+        groupForDatabase( "HTML Settings" ),
+        5 )
+property_ref_(
+        HTMLSizes, setHTMLSizes, QString,
+        groupForDatabase( "HTML Settings" ),
+        STR("") )
+property_ref_(
+        HTMLIncludeSelections, setHTMLIncludeSelections, QString,
+        groupForDatabase( "HTML Settings" ),
+        STR("") )
 
 property_ref_( password, setPassword, QString, groupForDatabase( "Privacy Settings" ), STR("") + HTMLBaseDir() )
 
@@ -491,3 +484,8 @@ QRect SettingsData::windowGeometry( WindowType win ) const
 {
     return value( "Window Geometry", win, QRect(0,0,800,600) );
 }
+
+bool Settings::SettingsData::hasUntaggedCategoryFeatureConfigured() const
+{
+    return DB::ImageDB::instance()->categoryCollection()->categoryNames().contains( untaggedCategory() );
+}
